/* /////////////////////////////////////////////////////////////////////////////
 * File:        stlsoft_cccap_borland.h
 *
 * Purpose:     Compiler feature discrimination for Borland C/C++.
 *
 * Created:     7th February 2003
 * Updated:     21st August 2003
 *
 * Author:      Matthew Wilson, Synesis Software Pty Ltd.
 *
 * License:     (Licensed under the Synesis Software Standard Source License)
 *
 *              Copyright (C) 2002-2003, Synesis Software Pty Ltd.
 *
 *              All rights reserved.
 *
 *              www:        http://www.synesis.com.au/stlsoft
 *                          http://www.stlsoft.org/
 *
 *              email:      submissions@stlsoft.org  for submissions
 *                          admin@stlsoft.org        for other enquiries
 *
 *              Redistribution and use in source and binary forms, with or
 *              without modification, are permitted provided that the following
 *              conditions are met:
 *
 *              (i) Redistributions of source code must retain the above
 *              copyright notice and contact information, this list of
 *              conditions and the following disclaimer.
 *
 *              (ii) Any derived versions of this software (howsoever modified)
 *              remain the sole property of Synesis Software.
 *
 *              (iii) Any derived versions of this software (howsoever modified)
 *              remain subject to all these conditions.
 *
 *              (iv) Neither the name of Synesis Software nor the names of any
 *              subdivisions, employees or agents of Synesis Software, nor the
 *              names of any other contributors to this software may be used to
 *              endorse or promote products derived from this software without
 *              specific prior written permission.
 *
 *              This source code is provided by Synesis Software "as is" and any
 *              warranties, whether expressed or implied, including, but not
 *              limited to, the implied warranties of merchantability and
 *              fitness for a particular purpose are disclaimed. In no event
 *              shall the Synesis Software be liable for any direct, indirect,
 *              incidental, special, exemplary, or consequential damages
 *              (including, but not limited to, procurement of substitute goods
 *              or services; loss of use, data, or profits; or business
 *              interruption) however caused and on any theory of liability,
 *              whether in contract, strict liability, or tort (including
 *              negligence or otherwise) arising in any way out of the use of
 *              this software, even if advised of the possibility of such
 *              damage.
 *
 * ////////////////////////////////////////////////////////////////////////// */


#ifndef _STLSOFT_INCL_H_STLSOFT
 #error This file must not be included independently of stlsoft.h
#endif /* !_STLSOFT_INCL_H_STLSOFT */

#ifdef _STLSOFT_INCL_H_STLSOFT_CCCAP_BORLAND
 #error This file cannot be included more than once in any compilation unit
#endif /* _STLSOFT_INCL_H_STLSOFT_CCCAP_BORLAND */

/* ////////////////////////////////////////////////////////////////////////// */

#define _STLSOFT_VER_H_STLSOFT_CCCAP_BORLAND_MAJOR      1
#define _STLSOFT_VER_H_STLSOFT_CCCAP_BORLAND_MINOR      4
#define _STLSOFT_VER_H_STLSOFT_CCCAP_BORLAND_REVISION   3
#define _STLSOFT_VER_H_STLSOFT_CCCAP_BORLAND_EDIT       18

/* /////////////////////////////////////////////////////////////////////////////
 * Compiler features
 */

/* Types:
 */

/* bool */
  #define __STLSOFT_CF_NATIVE_BOOL_SUPPORT

/* wchar_t */
#ifdef _WCHAR_T_DEFINED
 #define __STLSOFT_CF_NATIVE_WCHAR_T_SUPPORT
#else
 /* Not defined */
#endif /* _WCHAR_T_DEFINED */

 /* Native 8-bit integer */
 #define __STLSOFT_CF_NATIVE_8BIT_INT_SUPPORT
 #define __STLSOFT_NATIVE_INT8_T             __int8
 #define __STLSOFT_NATIVE_SINT8_T     signed __int8
 #define __STLSOFT_NATIVE_UINT8_T   unsigned __int8

 /* Native 16-bit integer */
 #define __STLSOFT_CF_NATIVE_16BIT_INT_SUPPORT
 #define __STLSOFT_NATIVE_INT16_T            __int16
 #define __STLSOFT_NATIVE_SINT16_T    signed __int16
 #define __STLSOFT_NATIVE_UINT16_T  unsigned __int16

 /* Native 32-bit integer */
 #define __STLSOFT_CF_NATIVE_32BIT_INT_SUPPORT
 #define __STLSOFT_NATIVE_INT32_T            __int32
 #define __STLSOFT_NATIVE_SINT32_T    signed __int32
 #define __STLSOFT_NATIVE_UINT32_T  unsigned __int32

 /* Native 64-bit integer */
 #define __STLSOFT_CF_NATIVE___int64_SUPPORT

/* long long */
//#define __STLSOFT_CF_NATIVE_LONG_LONG_SUPPORT

// It transpires that Borland cannot elide the contents of the static 
// asserts, so we only do them in debug mode
#ifdef _DEBUG
 #define __STLSOFT_CF_STATIC_ASSERT_SUPPORT
#endif /* _DEBUG */

/* Exception support */
#define __STLSOFT_CF_EXCEPTION_SUPPORT

/*  */
//#define __STLSOFT_CF_FUNCTION_SIGNATURE_FULL_ARG_QUALIFICATION_REQUIRED

/* Namespace support */
#if __BORLANDC__ < 0x0550
 #define _STLSOFT_NO_NAMESPACES
#endif /* __BORLANDC__ < 0x0550 */

#define __STLSOFT_CF_NAMESPACE_SUPPORT


/* Template support */
#ifdef __TEMPLATES__
 #define __STLSOFT_CF_TEMPLATE_SUPPORT
#else
 /* Not defined */
#endif /* __TEMPLATES__ */

#define __STLSOFT_CF_EXCEPTION_SIGNATURE_SUPPORT

#define __STLSOFT_CF_THROW_BAD_ALLOC

#define __STLSOFT_CF_TEMPLATE_CLASS_DEFAULT_FUNDAMENTAL_ARGUMENT_SUPPORT

#define __STLSOFT_CF_TEMPLATE_CLASS_DEFAULT_CLASS_ARGUMENT_SUPPORT

#define __STLSOFT_CF_MEMBER_TEMPLATE_FUNCTION_SUPPORT

#define __STLSOFT_CF_MEMBER_TEMPLATE_CTOR_SUPPORT

#if __BORLANDC__ >= 0x0551
 #define __STLSOFT_CF_MEMBER_TEMPLATE_RANGE_METHOD_SUPPORT
#endif /* __BORLANDC__ >= 0x0560 */

#define __STLSOFT_CF_MEMBER_TEMPLATE_CLASS_SUPPORT

#define __STLSOFT_CF_TEMPLATE_SPECIALISATION_SYNTAX

#define __STLSOFT_CF_TEMPLATE_PARTIAL_SPECIALISATION_SUPPORT

#define __STLSOFT_CF_TEMPLATE_OUTOFCLASSFN_QUALIFIED_TYPE_SUPPORT

#define __STLSOFT_CF_std_NAMESPACE

#define __STLSOFT_CF_std_char_traits_AVAILABLE

#define __STLSOFT_CF_ALLOCATOR_ALLOCATE_HAS_HINT

#define __STLSOFT_CF_ALLOCATOR_DEALLOCATE_HAS_OBJECTCOUNT

#define __STLSOFT_CF_BIDIRECTIONAL_ITERATOR_SUPPORT

#define __STLSOFT_CF_EXPLICIT_KEYWORD_SUPPORT

#define __STLSOFT_CF_MUTABLE_KEYWORD_SUPPORT

#define __STLSOFT_CF_TYPENAME_PARAM_KEYWORD_SUPPORT

#define __STLSOFT_CF_TYPENAME_TYPE_KEYWORD_SUPPORT

#define __STLSOFT_CF_TYPENAME_TYPE_DEF_KEYWORD_SUPPORT

//#define __STLSOFT_CF_TYPENAME_TYPE_MIL_KEYWORD_SUPPORT

//#define __STLSOFT_CF_MOVE_CONSTRUCTOR_SUPPORT

#define __STLSOFT_CF_KOENIG_LOOKUP_SUPPORT

#define __STLSOFT_CF_TEMPLATE_TEMPLATE_SUPPORT

#if __BORLANDC__ >= 0x0560
// #define __STLSOFT_CF_STATIC_ARRAY_SIZE_DETERMINATION_SUPPORT
#endif /* __BORLANDC__ >= 0x0560 */

//#define __STLSOFT_CF_VENEER_SUPPORT

// Shims are supported
// #define __STLSOFT_CF_TEMPLATE_SHIMS_NOT_SUPPORTED

#define __STLSOFT_CF_NEGATIVE_MODULUS_POSITIVE_GIVES_NEGATIVE_RESULT

#if defined(_STLSOFT_CUSTOM_ASSERT)
 /* You have defined the pre-processor symbol _STLSOFT_CUSTOM_ASSERT,
  * which stipulates that you will be providing your own assert. This
  * requires that you have defined _STLSOFT_CUSTOM_ASSERT() as a macro
  * taking 1 parameter (the condition to assert).
  *
  * Suppose you have a function _DisplayAssert(), which has the
  * following signature:
  *
  *   void _DisplayAssert(char const *file, int line, char const *expression);
  *
  * Presumably you would also have your own assert macro, say MY_ASSERT(),
  * defined as:
  *
  *   #define MY_ASSERT(_x) ((void)((!(_x)) ? ((void)(_DisplayAssert(__FILE__, __LINE__, #_x))) : ((void)0)))
  *
  * so you would simply need to define _STLSOFT_CUSTOM_ASSERT() in terms of
  * MY_ASSERT(), as in:
  *
  *  #define _STLSOFT_CUSTOM_ASSERT(_x)    MY_ASSERT(_x)
  *
  * where
  */
 #define __STLSOFT_CF_ASSERT_SUPPORT
 #define stlsoft_assert(_x)             _STLSOFT_CUSTOM_ASSERT(_x)
 #if defined(_STLSOFT_CUSTOM_ASSERT_INCLUDE)
  #define   __STLSOFT_CF_ASSERT_INCLUDE_NAME    _STLSOFT_CUSTOM_ASSERT_INCLUDE
 #else
  #error You must define _STLSOFT_CUSTOM_ASSERT_INCLUDE along with _STLSOFT_CUSTOM_ASSERT()
 #endif /* !_STLSOFT_CUSTOM_ASSERT_INCLUDE */
#else
 #define __STLSOFT_CF_ASSERT_SUPPORT
 //#define   __STLSOFT_CF_USE_cassert
 #define __STLSOFT_CF_ASSERT_INCLUDE_NAME    <assert.h>
 #define stlsoft_assert(_x)                  assert(_x)
#endif /* _STLSOFT_CUSTOM_ASSERT */

/* /////////////////////////////////////////////////////////////////////////////
 * Compiler warning suppression
 */

#if __BORLANDC__ >= 0x0550

 #if __BORLANDC__ < 0x0560

  /* Borland: "Condition is always false in function ..." */
  #pragma warn -8008

  /* Borland: "Unreachable code in function ..." */
  #pragma warn -8066

 #endif /* __BORLANDC__ < 0x0560 */

 /* Borland: "Functions with exception specifications are not expanded inline" */
 #pragma warn -8026

 /* Borland: "Functions containing reserved words are not expanded inline" */
 #pragma warn -8027

#endif /* __BORLANDC__ >= 0x0550 */

/* ////////////////////////////////////////////////////////////////////////// */
