/* /////////////////////////////////////////////////////////////////////////////
 * File:        stlsoft_constraints.h (originally MTAlgo.h, ::SynesisStl)
 *
 * Purpose:     Compile-time template constraints templates.
 *
 * Created:     19th November 1998
 * Updated:     29th July 2003
 *
 * Author:      Matthew Wilson, Synesis Software Pty Ltd.
 *
 * License:     (Licensed under the Synesis Software Standard Source License)
 *
 *              Copyright (C) 2002-2003, Synesis Software Pty Ltd.
 *
 *              All rights reserved.
 *
 *              www:        http://www.synesis.com.au/stlsoft
 *                          http://www.stlsoft.org/
 *
 *              email:      submissions@stlsoft.org  for submissions
 *                          admin@stlsoft.org        for other enquiries
 *
 *              Redistribution and use in source and binary forms, with or
 *              without modification, are permitted provided that the following
 *              conditions are met:
 *
 *              (i) Redistributions of source code must retain the above
 *              copyright notice and contact information, this list of
 *              conditions and the following disclaimer.
 *
 *              (ii) Any derived versions of this software (howsoever modified)
 *              remain the sole property of Synesis Software.
 *
 *              (iii) Any derived versions of this software (howsoever modified)
 *              remain subject to all these conditions.
 *
 *              (iv) Neither the name of Synesis Software nor the names of any
 *              subdivisions, employees or agents of Synesis Software, nor the
 *              names of any other contributors to this software may be used to
 *              endorse or promote products derived from this software without
 *              specific prior written permission.
 *
 *              This source code is provided by Synesis Software "as is" and any
 *              warranties, whether expressed or implied, including, but not
 *              limited to, the implied warranties of merchantability and
 *              fitness for a particular purpose are disclaimed. In no event
 *              shall the Synesis Software be liable for any direct, indirect,
 *              incidental, special, exemplary, or consequential damages
 *              (including, but not limited to, procurement of substitute goods
 *              or services; loss of use, data, or profits; or business
 *              interruption) however caused and on any theory of liability,
 *              whether in contract, strict liability, or tort (including
 *              negligence or otherwise) arising in any way out of the use of
 *              this software, even if advised of the possibility of such
 *              damage.
 *
 * ////////////////////////////////////////////////////////////////////////// */


#ifndef _STLSOFT_INCL_H_CONSTRAINTS
#define _STLSOFT_INCL_H_CONSTRAINTS

#ifndef __STLSOFT_DOCUMENTATION_SKIP_SECTION
#define _STLSOFT_VER_H_CONSTRAINTS_MAJOR      1
#define _STLSOFT_VER_H_CONSTRAINTS_MINOR      8
#define _STLSOFT_VER_H_CONSTRAINTS_REVISION   4
#define _STLSOFT_VER_H_CONSTRAINTS_EDIT       39
#endif /* !__STLSOFT_DOCUMENTATION_SKIP_SECTION */

/* /////////////////////////////////////////////////////////////////////////////
 * Includes
 */

#ifndef _STLSOFT_INCL_H_STLSOFT
 #include "stlsoft.h"   // Include the STLSoft root header
#endif /* !_STLSOFT_INCL_H_STLSOFT */

/* /////////////////////////////////////////////////////////////////////////////
 * Namespace
 */

#ifndef _STLSOFT_NO_NAMESPACE
namespace stlsoft
{
#endif /* _STLSOFT_NO_NAMESPACE */

/* /////////////////////////////////////////////////////////////////////////////
 * Macros
 */

#if defined(__STLSOFT_COMPILER_IS_DMC)
#define stlsoft_constraint_type_is_non_class_or_trivial_class(_T)           do { int i = sizeof(stlsoft_ns_qual(type_is_non_class_or_trivial_class)<_T>); } while(0)
#define stlsoft_constraint_variable_is_of_pointer_type_is_pointer(_T, _p)   do { int i = stlsoft_ns_qual(variable_is_of_pointer_type<_T>()(_p)); } while(0)
#else
#define stlsoft_constraint_type_is_non_class_or_trivial_class(_T)           stlsoft_static_assert(sizeof(stlsoft_ns_qual(type_is_non_class_or_trivial_class)<_T>) != 0)
//#define stlsoft_constraint_variable_is_of_pointer_type_is_pointer(_T, _p) stlsoft_static_assert(sizeof(stlsoft_ns_qual(variable_is_of_pointer_type)<_T>()(_p)) != 0)
#define stlsoft_constraint_variable_is_of_pointer_type_is_pointer(_T, _p)   do { int i = stlsoft_ns_qual(variable_is_of_pointer_type<_T>()(_p)); } while(0)
#endif /* __STLSOFT_COMPILER_IS_DMC */

/* /////////////////////////////////////////////////////////////////////////////
 * Constraints
 */

// has_base
//
// This structure constrains D to be convertible to B, ie
//

/// Constraint to ensure that the one type is convertible to another via inheritance
///
/// \param D The derived type
/// \param B The base type
template<   ss_typename_param_k D
        ,   ss_typename_param_k B
        >
struct has_base
{
public:
    ~has_base()
    {
        void(*p)(D*, B*) = constraints;
    }

private:
    static void constraints(D* pd, B* pb)
    {
        pb = pd;
    }
};

/// Constraint to ensure that the type is a <a href = "http://synesis.com.au/resources/articles/cpp/veneers.pdf">veneer</a>
///
/// \param T The parameterising type of the veneer
/// \param V The veneer type
template<   ss_typename_param_k T
        ,   ss_typename_param_k V
        >
struct is_veneer
{
    ~is_veneer()
    {
        void    (*pfn)(void) = constraints;

        ((void)pfn);
    }
private:
    static void constraints()
    {
        // The compiler will bring you here if T and V are not the same
        // size.
        int i[sizeof(T) == sizeof(V)];

        ((void)i);
    }
};


#ifndef __STLSOFT_DOCUMENTATION_SKIP_SECTION
// union TypeIsNonClassOrTrivialClass

// OBSOLETE
template <ss_typename_param_k T>
union TypeIsNonClassOrTrivialClass
{
    T   t;
    int i;

    TypeIsNonClassOrTrivialClass()  // ctor is required to satisy Metrowerks
    {}
};
#endif /* !__STLSOFT_DOCUMENTATION_SKIP_SECTION */

/// Constraint to ensure that a type is a built-in or trivial type.
///
/// \param T The type
// This class can be used to constrain a type to be of either built-in, e.g.
// int, or of a trivial type, i.e. aggregate types or types with publicly
// accessible default contructors and assignment operators
template <ss_typename_param_k T>
union type_is_non_class_or_trivial_class
{
    T   t;
    int i;

    type_is_non_class_or_trivial_class()  // ctor is required to satisy Metrowerks
    {}
};

/// Constraint to ensure that a type is a pointer type
///
/// \param T The type
template <ss_typename_param_k T>
struct variable_is_of_pointer_type
{
    size_t operator() (T const &t)
    {
        return sizeof(*t);
    }
};

/* ////////////////////////////////////////////////////////////////////////// */

#ifndef _STLSOFT_NO_NAMESPACE
} // namespace stlsoft
#endif /* _STLSOFT_NO_NAMESPACE */

/* ////////////////////////////////////////////////////////////////////////// */

#endif /* !_STLSOFT_INCL_H_CONSTRAINTS */

/* ////////////////////////////////////////////////////////////////////////// */
