/* /////////////////////////////////////////////////////////////////////////////
 * File:		perf_test.cpp
 *
 * Purpose:		Test program for the auto_buffer template.
 *
 * Created		
 * Updated:		 25th September 2003
 *
 * Copyright:	Synesis Software Pty Ltd, (c) 2003. All rights reserved.
 *
 * Home:		www.synesis.com.au/software
 *
 * ////////////////////////////////////////////////////////////////////////// */


#include <stdio.h>						//	fprintf
#include <stdlib.h>						//	atoi, alloca()
#include <malloc.h>						//	malloc & free, _alloca()
#include <new>							//	new & delete

#include <stlsoft.h>
#include <stlsoft_auto_buffer.h>		//	stlsoft's auto_buffer
#include <stlsoft_malloc_allocator.h>	//	stlsoft's malloc_allocator

stlsoft_ns_using(malloc_allocator)
stlsoft_ns_using(auto_buffer)

#if defined(WIN32) || \
    defined(_WIN32)

#if defined(__STLSOFT_COMPILER_IS_BORLAND)
 #include <winstl_tick_counter.h>

 typedef winstl_ns_qual(tick_counter)			performance_counter;
#else
 #include <winstl_performance_counter.h>

 typedef winstl_ns_qual(performance_counter)	performance_counter;
#endif /* compiler */

#else // Otherwise assume UNIX

 #include <unixstl_performance_counter.h>

 typedef unixstl_ns_qual(performance_counter)	performance_counter;

#endif /* WIN32 || _WIN32 */

#ifndef __STLSOFT_COMPILER_IS_WATCOM
 #include <vector>

 stlsoft_ns_using_std(vector)
#endif /* !__STLSOFT_COMPILER_IS_WATCOM */

/* /////////////////////////////////////////////////////////////////////////////
 * alloca?
 */

#ifdef TEST_ALLOCA
# undef TEST_ALLOCA
#endif /* TEST_ALLOCA */

#if defined(__STLSOFT_COMPILER_IS_BORLAND) || \
    defined(__STLSOFT_COMPILER_IS_COMO) || \
    defined(__STLSOFT_COMPILER_IS_DMC) || \
    defined(__STLSOFT_COMPILER_IS_GCC) || \
    defined(__STLSOFT_COMPILER_IS_INTEL) || \
    defined(__STLSOFT_COMPILER_IS_MWERKS) || \
    defined(__STLSOFT_COMPILER_IS_MSVC) || \
    defined(__STLSOFT_COMPILER_IS_WATCOM)
 #define TEST_ALLOCA
#endif /* compiler */

/* /////////////////////////////////////////////////////////////////////////////
 * VLA?
 */

#ifdef TEST_VLA
# undef TEST_VLA
#endif /* TEST_VLA */

#if defined(__STLSOFT_COMPILER_IS_COMO) || \
    defined(__STLSOFT_COMPILER_IS_DMC) || \
    defined(__STLSOFT_COMPILER_IS_GCC)
 #define TEST_VLA
#endif /* compiler */

/* /////////////////////////////////////////////////////////////////////////////
 * printf() format string
 */

#if defined(__STLSOFT_COMPILER_IS_BORLAND) || \
    defined(__STLSOFT_COMPILER_IS_GCC) || \
    defined(__STLSOFT_COMPILER_IS_INTEL) || \
    defined(__STLSOFT_COMPILER_IS_MSVC) || \
    defined(__STLSOFT_COMPILER_IS_WATCOM)
 #define test_printf_fmt	"%I64d"
#else
 #define test_printf_fmt	"%lld"
#endif /* compiler */

/* /////////////////////////////////////////////////////////////////////////////
 * Constants and definitions
 */

const int		WARMUPS						=	2;
#if 0
const size_t	AUTO_BUFFER_INTERNAL_SIZE	=	64;
#else
const size_t	AUTO_BUFFER_INTERNAL_SIZE	=	512;
#endif /* 0 */

/* /////////////////////////////////////////////////////////////////////////////
 * Typedefs
 */

typedef union Automatic
{
	char	_100[100];
	char	_1000[1000];
	char	_10000[10000];

} Automatic;

/* /////////////////////////////////////////////////////////////////////////////
 * Globals
 */

//extern volatile char	**s_p	=	0;

/* /////////////////////////////////////////////////////////////////////////////
 * Forward declarations
 */

int test_automatic(size_t alloc_size, int iterations);
int test_malloc(size_t alloc_size, int iterations);
int test_new(size_t alloc_size, int iterations);
int test_alloca(size_t alloc_size, int iterations, bool bTryAllocA);
int test_vector(size_t alloc_size, int iterations);
int test_vla(size_t alloc_size, int iterations, bool bVLA);
int test_auto_buffer(size_t alloc_size, int iterations);

int main(int argc, char **argv)
{
	int	iRet = 0;

	fprintf(stdout, "auto_buffer<> test: %s\n", __STLSOFT_COMPILER_LABEL_STRING);

	if(argc < 3)
	{
usage:
		fprintf(stderr, "  USAGE: Listing4 [-a] [-v] <alloc-size> <iterations>\n");
		fprintf(stderr, "    -a              -   do an alloca() test (likely to\n");
		fprintf(stderr, "                    -     crash with insufficient stack)\n");
		fprintf(stderr, "    -v              -   do a VLA test (likely to\n");
		fprintf(stderr, "                    -     crash with insufficient stack)\n");
		fprintf(stderr, "    <alloc-size>    -   the amount of memory to allocate\n");
		fprintf(stderr, "    <iterations>    -   number of times to execute the test loop\n");

		iRet = 1;
	}
	else
	{
		char const	*pcszAllocSize	=	NULL;
		char const	*pcszIterations	=	NULL;
		bool		bTryAllocA		=	false;
		bool		bVLA			=	false;

		for(int i = 1; i < argc; ++i)
		{
			char const	*arg	=	argv[i];

			if(arg[0] == '-')
			{
				switch(arg[1])
				{
					case	'a':
						bTryAllocA = true;
						break;
					case	'v':
						bVLA = true;
						break;
					default:
						goto usage;
				}
			}
			else
			{
				if(NULL == pcszAllocSize)
				{
					pcszAllocSize = arg;
				}
				else if(NULL == pcszIterations)
				{
					pcszIterations = arg;
				}
			}
		}

#ifdef _DEBUG
		puts(pcszAllocSize);
		puts(pcszIterations);
#endif /* _DEBUG */

		size_t	alloc_size	=	static_cast<size_t>(atoi(pcszAllocSize));
		int		iterations	=	atoi(pcszIterations);

		fprintf(stdout, "internal size: %d; block size: %d\n", __STLSOFT_COMPILER_LABEL_STRING, alloc_size);

#ifdef WIN32
		::SetThreadPriority(::GetCurrentThread(), THREAD_PRIORITY_TIME_CRITICAL);
#endif // WIN32

		if(alloc_size > sizeof(Automatic))
		{
			fprintf(stderr, "Size of buffer allocation cannot be larger than %d for this implementation!\n", sizeof(Automatic));
		}
		else
		{
			iRet += test_automatic(alloc_size, iterations);
			iRet += test_malloc(alloc_size, iterations);
			iRet += test_new(alloc_size, iterations);
			iRet += test_vector(alloc_size, iterations);
			iRet += test_auto_buffer(alloc_size, iterations);
			iRet += test_alloca(alloc_size, iterations, bTryAllocA);
			iRet += test_vla(alloc_size, iterations, bVLA);
#if 1
			fprintf(stdout, "Total: %d\n", iRet);
#endif /* 0 */
		}
	}

	return iRet;
}


/* /////////////////////////////////////////////////////////////////////////////
 * Test functions
 */

int test_automatic(size_t alloc_size, int iterations)
{
	int					i;
	int					j;
	int					iRet;
	performance_counter	counter;

	fprintf(stdout, "automatic:     ");

	for(j = 0; j < WARMUPS; ++j)
	{
		counter.start();
		for(i = 0, iRet = 0; i < iterations; ++i)
		{
			// (i) Allocate
			Automatic	_buffer;
			char		*buffer	=	_buffer._1000;

			// (ii) Use
//			s_p = const_cast<volatile char**>(&buffer);

			// Help to avoid compiler optimising away our code, by touching 
			// last element in the array ...
			buffer[alloc_size - 1] += i;
			// ... and testing the next to last one
			iRet ^= buffer[alloc_size - 1];

			// (iii) Deallocate
			; // Nothing to do
		}
		counter.stop();
	}

	fprintf(stdout, " " test_printf_fmt "\n", counter.get_milliseconds());

	return iRet;
}

int test_malloc(size_t alloc_size, int iterations)
{
	int					i;
	int					j;
	int					iRet;
	performance_counter	counter;

	fprintf(stdout, "malloc():      ");

	for(j = 0; j < WARMUPS; ++j)
	{
		counter.start();
		for(i = 0, iRet = 0; i < iterations; ++i)
		{
			// (i) Allocate
			char	*buffer	=	(char*)malloc(alloc_size);

			// (ii) Use
//			s_p = const_cast<volatile char**>(&buffer);

			// Help to avoid compiler optimising away our code, by touching 
			// last element in the array ...
			buffer[alloc_size - 1] += i;
			// ... and testing the next to last one
			iRet ^= buffer[alloc_size - 1];

			// (iii) Deallocate
			free(buffer);
		}
		counter.stop();
	}

	fprintf(stdout, " " test_printf_fmt "\n", counter.get_milliseconds());

	return iRet;
}

int test_new(size_t alloc_size, int iterations)
{
	int					i;
	int					j;
	int					iRet;
	performance_counter	counter;

	fprintf(stdout, "operator new:  ");

	for(j = 0; j < WARMUPS; ++j)
	{
		counter.start();
		for(i = 0, iRet = 0; i < iterations; ++i)
		{
			// (i) Allocate
			char	*buffer	=	new char[alloc_size];

			// (ii) Use
//			s_p = const_cast<volatile char**>(&buffer);

			// Help to avoid compiler optimising away our code, by touching 
			// last element in the array ...
			buffer[alloc_size - 1] += i;
			// ... and testing the next to last one
			iRet ^= buffer[alloc_size - 1];

			// (iii) Deallocate
			delete [] buffer;
		}
		counter.stop();
	}

	fprintf(stdout, " " test_printf_fmt "\n", counter.get_milliseconds());

	return iRet;
}

int test_alloca(size_t alloc_size, int iterations, bool bTryAllocA)
{
#ifdef TEST_ALLOCA
	if(!bTryAllocA)
#endif /* !TEST_ALLOCA */
	{
		return fprintf(stdout, "alloca():       -\n");
	}

#ifdef TEST_ALLOCA
	int					i;
	int					j;
	int					iRet;
	performance_counter	counter;

	fprintf(stdout, "alloca():      ");
	fflush(stdout);

	for(j = 0; j < WARMUPS; ++j)
	{
		counter.start();
		for(i = 0, iRet = 0; i < iterations; ++i)
		{
			// (i) Allocate
#ifdef _MSC_VER
			char	*buffer	=	(char*)_alloca(alloc_size);
#elif defined(__STLSOFT_COMPILER_IS_GCC)
			char	*buffer	=	(char*)__builtin_alloca(alloc_size);
#else
			char	*buffer	=	(char*)alloca(alloc_size);
#endif // _MSC_VER

			// (ii) Use
//			s_p = const_cast<volatile char**>(&buffer);

			// Help to avoid compiler optimising away our code, by touching 
			// last element in the array ...
			buffer[alloc_size - 1] += i;
			// ... and testing the next to last one
			iRet ^= buffer[alloc_size - 1];

			// (iii) Deallocate
			; // Nothing to do
		}
		counter.stop();
	}

	fprintf(stdout, " " test_printf_fmt "\n", counter.get_milliseconds());

	return iRet;
#endif /* TEST_ALLOCA */
}

int test_vector(size_t alloc_size, int iterations)
{
#ifdef __STLSOFT_COMPILER_IS_WATCOM
	return fprintf(stdout, "vector<>:      \n");
#else
#ifdef __STLSOFT_COMPILER_IS_DMC
	typedef	vector<char>							vector_t;
#else
	typedef	vector<char, malloc_allocator<char> >	vector_t;
#endif /* __STLSOFT_COMPILER_IS_DMC */

	int					i;
	int					j;
	int					iRet;
	performance_counter	counter;

	fprintf(stdout, "vector<>:      ");

	for(j = 0; j < WARMUPS; ++j)
	{
		counter.start();
		for(i = 0, iRet = 0; i < iterations; ++i)
		{
			// (i) Allocate
			vector_t	buffer(alloc_size);

			// (ii) Use
//			s_p = const_cast<volatile char**>(&buffer);

			// Help to avoid compiler optimising away our code, by touching 
			// last element in the array ...
			buffer[alloc_size - 1] += i;
			// ... and testing the next to last one
			iRet ^= buffer[alloc_size - 1];

			// (iii) Deallocate
			; // Destructor will handle
		}
		counter.stop();
	}

	fprintf(stdout, " " test_printf_fmt "\n", counter.get_milliseconds());

	return iRet;
#endif /* __STLSOFT_COMPILER_IS_WATCOM */
}

int test_vla(size_t alloc_size, int iterations, bool bVLA)
{
#ifdef TEST_VLA
	if(!bVLA)
#endif /* TEST_VLA */
	{
		return fprintf(stdout, "VLA:            -\n");
	}

#ifdef TEST_VLA
	int					i;
	int					j;
	int					iRet;
	performance_counter	counter;

	fprintf(stdout, "VLA:           ");
	fflush(stdout);

	for(j = 0; j < WARMUPS; ++j)
	{
		counter.start();
		for(i = 0, iRet = 0; i < iterations; ++i)
		{
			// (i) Allocate
			char	buffer[alloc_size];

			// (ii) Use
//			s_p = const_cast<volatile char**>(&buffer);

			// Help to avoid compiler optimising away our code, by touching 
			// last element in the array ...
			buffer[alloc_size - 1] += i;
			// ... and testing the next to last one
			iRet ^= buffer[alloc_size - 1];

			// (iii) Deallocate
			; // Nothing to do
		}
		counter.stop();
	}

	fprintf(stdout, " " test_printf_fmt "\n", counter.get_milliseconds());

	return iRet;
#else
	((void)alloc_size);
	((void)iterations);
	((void)bVLA);

	return 0;
#endif /* TEST_VLA */
}

int test_auto_buffer(size_t alloc_size, int iterations)
{
	typedef	auto_buffer<char, malloc_allocator<char>, AUTO_BUFFER_INTERNAL_SIZE>	buffer_t;

	int					i;
	int					j;
	int					iRet;
	performance_counter	counter;

	fprintf(stdout, "auto_buffer<>: ");

	for(j = 0; j < WARMUPS; ++j)
	{
		counter.start();
		for(i = 0, iRet = 0; i < iterations; ++i)
		{
			// (i) Allocate
			buffer_t	buffer(alloc_size);

			// (ii) Use
//			s_p = const_cast<volatile char**>(&buffer);

			// Help to avoid compiler optimising away our code, by touching 
			// last element in the array ...
			buffer[alloc_size - 1] += i;
			// ... and testing the next to last one
			iRet ^= buffer[alloc_size - 1];

			// (iii) Deallocate
			; // Destructor will handle
		}
		counter.stop();
	}

	fprintf(stdout, " " test_printf_fmt "\n", counter.get_milliseconds());

	return iRet;
}

/* ////////////////////////////////////////////////////////////////////////// */
