//----------------------------------------------------------------------------
// File:  static_type_inference.hh 
// Date:  Fri Mar 21 18:29:53 PST 2003 
// Author: Martin Casado
//
// Description:
//
// Abandon all hope ye' who enter here .. template wasteland! 
//
// ----------------------------------------------------------------------------
//
//    Copyright 2003. The Regents of the University of California.
//    All rights reserved.
//
//    This work was produced at the University of California, Lawrence Livermore
//    National Laboratory (UC LLNL) under contract no. W-7405-ENG-48 (Contract
//    48) between the U.S. Department of Energy (DOE) and The Regents of the
//    University of California (University) for the operation of UC LLNL.
//    Copyright is reserved to the University for purposes of controlled
//    dissemination, commercialization through formal licensing, or other
//    disposition under terms of Contract 48; DOE policies, regulations and
//    orders; and U.S. statutes. The rights of the Federal Government are
//    reserved under Contract 48 subject to the restrictions agreed upon by the
//    DOE and University as allowed under DOE Acquisition Letter 91-7.
//
//    DISCLAIMER
//    This software was prepared as an account of work sponsored by an
//    agency of the United States Government. Neither the United States
//    Government nor the University of California nor any of their employees,
//    makes any warranty, express or implied, or assumes any liability or
//    responsibility for the accuracy, completeness, or usefulness of any
//    information, apparatus, product, or process disclosed, or represents that
//    its use would not infringe protectedly-owned rights. Reference herein to any
//    specific commercial products, process, or service by trade name, trademark,
//    manufacturer, or otherwise, does not necessarily constitute or imply its
//    endorsement, recommendation, or favoring by the United States Government or
//    the University of California. The views and opinions of authors expressed
//    herein do not necessarily represent those of the United States
//    Government or the University of California, and shall not be used for
//    advertising or product endorsement purposes.
//
//-----------------------------------------------------------------------------

#ifndef PACMAN_UTIL_HH
#define PACMAN_UTIL_HH

#include <iostream>

#include "empty.hh"

#include "boost/type_traits/is_base_and_derived.hpp"

extern "C"
{
#include "stdint.h"
}

namespace pacman
{

//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------

struct ip_proto
{ static const uint8_t IPPROTO = 0; };

struct icmp_proto
{ static const uint8_t ICMPTYPE = 0; };

struct ether_type
{ static const uint8_t ETHTYPE = 0; };

struct ip_port
{ static const uint8_t IP_PORT = 0; };

struct arp_hardware
{ static const uint16_t ARPTYPE = 0; };

struct arp_protocol
{ static const uint16_t ARPTYPE = 0; };

//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
template <bool expression, typename True, typename False>
struct type_if_true_else
{ // -- true
    typedef True Type; 
};

template <typename True, typename False>
struct type_if_true_else <false, True, False>
{ // -- false
    typedef False Type; 
};
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
template <typename T>
struct empty_aware_sizeof
{
    static const uint32_t val = sizeof(T);
};
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
template <>
struct empty_aware_sizeof <empty>
{
    static const uint32_t val = 0; 
};
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
template <typename T> struct szof;
template <>
struct szof<empty>
{
	static const uint8_t len = 0;
};
template <typename T>
struct szof
{
	static const uint8_t len = sizeof(T);
};
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
template <typename T>
struct hdr
{
    static const uint32_t len = no_hdr_len_defined_for_this_type;
};
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// 1 Parameter
//-----------------------------------------------------------------------------
template <>
struct hdr <empty>
{
    static const uint32_t len = 0;
};
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
template <template <typename L> class HDR, typename L>
struct hdr <HDR<L> >
{
    static const uint32_t len = sizeof(HDR<L>) - empty_aware_sizeof<L>::val;
};
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// 2 Parameters
//-----------------------------------------------------------------------------
template <template <typename O, typename L> class HDR, typename O, typename L>
struct hdr <HDR<O,L> >
{
    static const uint32_t len = sizeof(HDR<O,L>) - empty_aware_sizeof<L>::val;
};
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// 3 Parameters
//-----------------------------------------------------------------------------
template <template <uint8_t, typename O, typename T> class HDR, uint8_t F, typename O, typename L>
struct hdr <HDR<F,O,L> >
{
    static const uint32_t len = sizeof(HDR<F,O,L>) - empty_aware_sizeof<L>::val;
};
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
template <typename T>
inline
uint8_t hdrlen(T& in)
{
    return hdr<T>::len;
}
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
template <typename T>
struct just_header
{ // false
    static const bool val = false;
};
template <template  <class> class T>
struct just_header <T<empty> >   
{
    static const bool val = true;
};
template <template  <class, class> class T, class T2>
struct just_header <T<T2, empty> >   
{
    static const bool val = true;
};
template <template  <class, class, class> class T, class T2, class T3>
struct just_header <T<T2, T3, empty> >   
{
    static const bool val = true;
};
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
template <typename T>
inline
bool is_just_header(T& in)
{
    return just_header<T>::val;
}
//-----------------------------------------------------------------------------


//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
template <bool B, typename T,  uint8_t default_>
struct ip_proto_if_true
{ // -- false
    static const uint8_t val = default_;
};
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
template <typename T, uint8_t default_>
struct ip_proto_if_true <true, T, default_>
{ // -- true
    static const uint8_t val = T::IPPROTO;
};
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
template <typename T, uint8_t default_>
struct get_ip_proto
{
    static const uint8_t val = ip_proto_if_true <
                             boost::is_base_and_derived<ip_proto,T>::value,
                             T,
                             default_
                             >::val;
};
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
template <bool B, typename T,  uint8_t default_>
struct icmp_type_if_true
{ // -- false
    static const uint8_t val = default_;
};
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
template <typename T, uint8_t default_>
struct icmp_type_if_true <true, T, default_>
{ // -- true
    static const uint8_t val = T::ICMPTYPE;
};
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
template <typename T, uint8_t default_>
struct get_icmp_type
{
    static const uint8_t val = icmp_type_if_true <
                             boost::is_base_and_derived<icmp_proto,T>::value,
                             T,
                             default_
                             >::val;
};
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
template <bool B, typename T,  uint16_t default_>
struct ether_type_if_true
{ // -- false
    static const uint16_t val = default_;
};
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
template <typename T, uint16_t default_>
struct ether_type_if_true <true, T, default_>
{ // -- true
    static const uint16_t val = T::ETHTYPE;
};
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
template <typename T, uint16_t default_>
struct get_ether_type 
{
    static const uint16_t val = ether_type_if_true <
                             boost::is_base_and_derived<ether_type,T>::value,
                             T,
                             default_
                             >::val;
};
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
template <bool B, typename T,  uint8_t default_>
struct ip_port_if_true
{ // -- false
    static const uint8_t val = default_;
};
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
template <typename T, uint8_t default_>
struct ip_port_if_true <true, T, default_>
{ // -- true
    static const uint8_t val = T::IP_PORT;
};
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
template <typename T, uint8_t default_>
struct get_ip_port
{
    static const uint8_t val = ip_port_if_true <
                             boost::is_base_and_derived<ip_port,T>::value,
                             T,
                             default_
                             >::val;
};
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------


//-----------------------------------------------------------------------------
template <uint16_t in>
struct htons_
{
        static const uint16_t val = (in<<8&0xff00)|(in>>8&0x00ff); 
};
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
template <uint32_t in>
struct htonl_
{
        static const uint32_t val =
            (in<<24&0xff000000)|
            (in<<8 &0x00ff0000)|
            (in>>8 &0x0000ff00)|
            (in>>24&0x000000ff); 
};
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
template <typename T>
struct get_payload
{ // -- not a template
    static const T& func(T& in)
    { return    in; }
};
template <template <class> class T, class T2>
struct get_payload <T<T2> >
{ // -- template  with payload
    static const T2& func(T<T2>& in)
    { return    in.data; }
};
template <template <class, class> class T, class T2, class T3>
struct get_payload <T<T2, T3> >
{ // -- two parameter template  with payload
    static const T3& func(T<T2, T3>& in)
    { return    in.data; }
};
template <template <class> class T>
struct get_payload <T<empty> >
{ // -- template without payload
    static const T<empty>& func(T<empty>& in)
    { return    in; }
};
template <template <class, class> class T, class T2>
struct get_payload <T<T2, empty> >
{ // -- two argument template without payload
    static const T<T2, empty>& func(T<T2, empty>& in)
    { return    in; }
};
//-----------------------------------------------------------------------------

};

#endif // --  PACMAN_UTIL_HH
