#include <OB\CORBA.h>
#include "test.h"
#include <iostream>
#include <fstream>
#include <time.h>

using namespace std;

int run(CORBA::ORB_ptr orb, const char *refname, int iterations, int msgtype)
{
     char ref[2048];
     {
          ifstream reffile;
          reffile.open(refname);

          if (!reffile.is_open())
          {
               cout << "cannot open IOR file " << endl;
               return EXIT_FAILURE;
          }

          reffile >> ref;
     }

     CORBA::Object_var obj = orb->string_to_object(ref);
     test_var s = test::_narrow(obj);

     CORBA::Long result;

     cout << "forcing one message" << endl;
     if (msgtype == 1)
          s->doit(7);     // one-way
     else
          result = s->add(7, 8);     // two-way

     cout << "running the loop" << endl;

     time_t t1 = time(NULL);

     for (int i = 0; i < iterations; ++i)
     {
          if (msgtype == 1)
               s->doit(7);
          else
               result = s->add(7, 8);
     }

     time_t t2 = time(NULL);

     int t = (int)(t2 - t1);
     cout << "time : " << t << endl;
     if (t != 0)
          cout << "speed: " << (double)(iterations / t) << " msg/sec" << endl;
     else
          cout << "speed: cannot be computed" << endl;

     return EXIT_SUCCESS;
}

int main(int argc, char *argv[])
{
     if (argc != 4)
     {
          cout << "usage: client reffilename iterations oneway/twoway" << endl;
          return EXIT_FAILURE;
     }

     const char *reffile = argv[1];
     int iterations;
     {
          istringstream ss(argv[2]);
          ss >> iterations;
     }
     string mtype = argv[3];
     int msgtype;
     if (mtype == "oneway")
     {
          msgtype = 1;
          cout << "one-way messages" << endl;
     }
     else 
     {
          msgtype = 2;
          cout << "two-way messages" << endl;
     }
     
     cout << "starting client" << endl;

     int status = EXIT_SUCCESS;
     CORBA::ORB_var orb;

     try
     {
          cout << "initializing ORB" << endl;
          orb = CORBA::ORB_init(argc, argv);
          status = run(orb, reffile, iterations, msgtype);
     }
     catch (const CORBA::Exception &)
     {
          cout << "exception caught" << endl;
          status = EXIT_FAILURE;
     }

     if (!CORBA::is_nil(orb))
     {
          try
          {
               cout << "destroying ORB" << endl;
               orb->destroy();
          }
          catch (const CORBA::Exception &)
          {
               cout << "exception caught" << endl;
               status = EXIT_FAILURE;
          }
     }

     return status;
}
