#include "yamic.h"
#include "yamiparams.h"
#include "yamierrors.h"

#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <string.h>

const int  serverport   = 12340;
const int  clientport   = 12341;
const char *domainname  = "someDomain";
const char *objectname  = "object";
const char *twowaymsg   = "add";


int send_two_way_message(HYAMIAGENT agent)
{
     HPARAMSET paramset, returnparamset;
     HMESSAGE hm;
     enum msgStatus status;
     int a, b, result;
     int cc;
     
     cc = yamiCreateParamSet(&paramset, 2);
     if (cc != YAMIOK)
     {
          printf("cannot create parameter set: %s\n", yamiErrorString(cc));
          return cc;
     }
     
     a = 7;
     b = 8;
     
     cc = yamiSetInt(paramset, 0, a);
     if (cc != YAMIOK)
     {
          printf("cannot set a value: %s\n", yamiErrorString(cc));
          return cc;
     }
          
     cc = yamiSetInt(paramset, 1, b);
     if (cc != YAMIOK)
     {
          printf("cannot set a value: %s\n", yamiErrorString(cc));
          return cc;
     }
          
     cc = yamiAgentMsgSend(agent, domainname, objectname, twowaymsg,
          paramset, &hm);
     if (cc != YAMIOK)
     {
          printf("cannot send the message: %s\n", yamiErrorString(cc));
          return cc;
     }
          
     cc = yamiDestroyParamSet(paramset);
     if (cc != YAMIOK)
     {
          printf("cannot destroy the parameter set: %s\n",
               yamiErrorString(cc));
          return cc;
     }

     /* do blocking wait */
     
     cc = yamiAgentProcessEvent(agent, -1);
     if (cc != YAMIOK)
     {
          printf("error while processing the event: %s\n",
               yamiErrorString(cc));
          return cc;
     }
     
     cc = yamiAgentMsgGetStatus(hm, &status);
     if (cc != YAMIOK)
     {
          printf("cannot retrieve status of the messagege: %s\n",
               yamiErrorString(cc));
          return cc;
     }
     
     if (status == eReplied)
     {
          cc = yamiAgentMsgGetResponse(hm, &returnparamset);
          if (cc != YAMIOK)
          {
               printf("cannot retrieve the return parameter set: %s\n",
                    yamiErrorString(cc));
               return cc;
          }
          
          cc = yamiGetInt(returnparamset, 0, &result);
          if (cc != YAMIOK)
          {
               printf("cannot get value from the parameter set: %s\n",
                    yamiErrorString(cc));
               return cc;
          }
          
          cc = yamiDestroyParamSet(returnparamset);
          if (cc != YAMIOK)
          {
               printf("cannot destroy the return parameter set: %s\n",
                    yamiErrorString(cc));
               return cc;
          }

          if (result != a + b)
          {
               puts("the result is incorrect");
               return -1;
          }
     }
     else
     {
          puts("the last message was not replied correctly");
          if (status == eTimedOut)
               puts("the message was TIMED OUT");
          return -1;
     }
     
     cc = yamiAgentMsgDestroy(hm);
     if (cc != YAMIOK)
     {
          printf("cannot destroy the message object: %s\n",
               yamiErrorString(cc));
          return cc;
     }
     
     return YAMIOK;
}

int main(int argc, char *argv[])
{
     struct yamipolicies policies;
     const char *serverhost;
     int iterations;
     HYAMIAGENT agent;
     time_t t1, t2;
     int t;
     int i, cc;
     
     if (argc != 3)
     {
          puts("usage: clievents hostname iterations");
          return -1;
     }
     
     serverhost = argv[1];
     iterations = atoi(argv[2]);
     
     puts("starting the client");
     
     cc = yamiNetInitialize();
     if (cc != YAMIOK)
     {
          printf("cannot initialize network: %s\n", yamiErrorString(cc));
          return -1;
     }
     
     /* note: policies are set up for the explicit event processing */
     
     yamiGetDefaultPolicies(&policies);
     
     policies.dispatchers = 0;
     policies.hassender   = 0;
     policies.reuseaddr   = 1;
     policies.hasreceiver = 0;
     policies.haswaker    = 0;
     policies.mtuser      = 0;
     
     cc = yamiCreateAgent(&agent, clientport, &policies);
     if (cc != YAMIOK)
     {
          printf("cannot create agent: %s\n", yamiErrorString(cc));
          return -1;
     }

     cc = yamiAgentDomainRegister(agent, domainname,
          serverhost, serverport, 2);
     if (cc != YAMIOK)
     {
          printf("cannot register domain: %s\n", yamiErrorString(cc));
          return -1;
     }

     puts("forcing one message for getting the connection");
     
     cc = send_two_way_message(agent);
     if (cc != YAMIOK)
     {
          yamiDestroyAgent(agent);
          return -1;
     }
     
     puts("running the loop...");

     t1 = time(NULL);
     
     for (i = 0; i < iterations; ++i)
     {
          cc = send_two_way_message(agent);
          if (cc != YAMIOK)
          {
               yamiDestroyAgent(agent);
               return -1;
          }
     }
     
     t2 = time(NULL);
     t = (int)(t2 - t1);
     
     printf("time : %d sec\n", t);

     if (t != 0)
     {
          printf("speed: %f msg/sec\n", (double)(iterations / t));
     }
     else
     {
          printf("speed: cannot be computed\n");
     }
     
     cc = yamiDestroyAgent(agent);
     if (cc != YAMIOK)
     {
          printf("cannot destroy the agent: %s\n", yamiErrorString(cc));
          return -1;
     }
     
     cc = yamiNetCleanup();
     if (cc != YAMIOK)
     {
          printf("cannot clean up the network%s\n", yamiErrorString(cc));
          return -1;
     }
     
     return 0;
}
