#include "yamic.h"
#include "yamiparams.h"
#include "yamierrors.h"

#include <stdio.h>
#include <string.h>

const int  serverport   = 12340;
const char *objectname  = "object";
const char *onewaymsg   = "doit";
const char *twowaymsg   = "add";

int onewaycounter;
int twowaycounter;

void servant(HINCMSG incomingmsg)
{
     HPARAMSET paramset, returnparamset;
     const char *name;
     int a, b, result;
     int cc;

     cc = yamiAgentIncomingMsgGetMsgName(incomingmsg, &name);
     if (cc != YAMIOK)
     {
          printf("cannot retrieve the msg name: %s\n", yamiErrorString(cc));
          return;
     }
     
     if (strcmp(name, onewaymsg) == 0)
     {
          cc = yamiAgentIncomingMsgGetParameters(incomingmsg, &paramset);
          if (cc != YAMIOK)
          {
               printf("cannot retrieve the parameter set: %s\n",
                    yamiErrorString(cc));
               return;
          }

          cc = yamiGetInt(paramset, 0, &a);
          if (cc != YAMIOK)
          {
               printf("cannot retrieve the value: %s\n", yamiErrorString(cc));
               return;
          }
          
          cc = yamiDestroyParamSet(paramset);
          if (cc != YAMIOK)
          {
               printf("cannot destroy the parameter set: %s\n",
                    yamiErrorString(cc));
               return;
          }
          
          cc = yamiAgentIncomingMsgEat(incomingmsg);
          if (cc != YAMIOK)
          {
               printf("cannot eat the message: %s\n",
                    yamiErrorString(cc));
               return;
          }

          ++onewaycounter;
     }
     else if (strcmp(name, twowaymsg) == 0)
     {
          cc = yamiAgentIncomingMsgGetParameters(incomingmsg, &paramset);
          if (cc != YAMIOK)
          {
               printf("cannot retrieve the parameter set: %s\n",
                    yamiErrorString(cc));
               return;
          }

          cc = yamiGetInt(paramset, 0, &a);
          if (cc != YAMIOK)
          {
               printf("cannot retrieve the value: %s\n", yamiErrorString(cc));
               return;
          }
          
          cc = yamiGetInt(paramset, 1, &b);
          if (cc != YAMIOK)
          {
               printf("cannot retrieve the value: %s\n", yamiErrorString(cc));
               return;
          }
          
          cc = yamiDestroyParamSet(paramset);
          if (cc != YAMIOK)
          {
               printf("cannot destroy the parameter set: %s\n",
                    yamiErrorString(cc));
               return;
          }

          result = a + b;

          cc = yamiCreateParamSet(&returnparamset, 1);
          if (cc != YAMIOK)
          {
               printf("cannot create the return parameter set: %s\n",
                    yamiErrorString(cc));
               return;
          }
          
          cc = yamiSetInt(returnparamset, 0, result);
          if (cc != YAMIOK)
          {
               printf("cannot set the result value: %s\n",
                    yamiErrorString(cc));
               return;
          }
          
          cc = yamiAgentIncomingMsgReply(incomingmsg, returnparamset);
          if (cc != YAMIOK)
          {
               printf("cannot reply to the message: %s\n",
                    yamiErrorString(cc));
               return;
          }
          
          cc = yamiDestroyParamSet(returnparamset);
          if (cc != YAMIOK)
          {
               printf("cannot destroy the return parameter set: %s\n",
                    yamiErrorString(cc));
               return;
          }
     
          ++twowaycounter;
     }
     else
     {
          puts("wrong msg name");
          return;
     }
}

#define BUFSIZE 100

int main()
{
     struct yamipolicies policies;
     HYAMIAGENT agent;
     char dummy[BUFSIZE];
     int cc;
     
     puts("starting the server");
     
     onewaycounter = 0;
     twowaycounter = 0;
     
     cc = yamiNetInitialize();
     if (cc != YAMIOK)
     {
          printf("cannot initialize network: %s\n", yamiErrorString(cc));
          return -1;
     }

     /* note: default policies are used, except reuseaddr */
     
     yamiGetDefaultPolicies(&policies);
     
     policies.reuseaddr   = 1;
     
     cc = yamiCreateAgent(&agent, serverport, &policies);
     if (cc != YAMIOK)
     {
          printf("cannot create the agent: %s\n", yamiErrorString(cc));
          return -1;
     }

     cc = yamiAgentObjectRegister(agent, objectname,
          passive_multithreaded, servant, NULL);
     if (cc != YAMIOK)
     {
          puts("cannot register the object");
          return cc;
     }
     
     puts("going to sleep...");
     puts("press enter to see current counter value (or type q to quit)");
     
     while (1)
     {
          fgets(dummy, BUFSIZE, stdin);
          if (dummy[0] == 'q')
               break;

          printf("one-way message counter: %d\n", onewaycounter);
          printf("two-way message counter: %d\n", twowaycounter);
     }
     
     puts("quitting");

     cc = yamiDestroyAgent(agent);
     if (cc != YAMIOK)
     {
          printf("cannot destroy the agent: %s\n", yamiErrorString(cc));
          return -1;
     }
     
     cc = yamiNetCleanup();
     if (cc != YAMIOK)
     {
          printf("cannot clean up the network: %s\n", yamiErrorString(cc));
          return -1;
     }
     
     return 0;
}
