#include "yamic.h"
#include "yamiparams.h"
#include "yamierrors.h"

#include <stdio.h>
#include <ctype.h>
#include <string.h>

const char *serverhost  = "127.0.0.1";
const int  serverport   = 12340;
const int  clientport   = 12341;
const char *domainname  = "someDomain";
const char *objectname  = "calculator";

#define BUFSIZE 100

int main()
{
     HYAMIAGENT agent;
     HPARAMSET paramset, returnparamset;
     HMESSAGE hm;
     enum msgStatus status;
     char buf[BUFSIZE];
     int val1, val2, result;
     char op;
     const char *msgname;
     int i, cc;
     
     puts("starting the client");
     
     cc = yamiNetInitialize();
     if (cc != YAMIOK)
     {
          puts("cannot initialize network");
          return 0;
     }
     
     cc = yamiCreateAgent(&agent, clientport, NULL);
     if (cc != YAMIOK)
     {
          puts("cannot create agent");
          printf("0x%08x\n", cc);
          return 0;
     }

     cc = yamiAgentDomainRegister(agent, domainname,
          serverhost, serverport, 2);
     if (cc != YAMIOK)
     {
          puts("cannot register domain");
          return 0;
     }

     while (1)
     {
          printf("please enter the expression (or \'quit\'): ");

          fgets(buf, BUFSIZE, stdin);
          for (i = strlen(buf) - 1; i >= 0; --i)
               if (isspace(buf[i]))
                    buf[i] = 0;
               else
                    break;

          if (strcmp(buf, "quit") == 0)
               break;

          if (sscanf(buf, "%d%c%d", &val1, &op, &val2) != 3)
          {
               puts("wrong expression");
               continue;
          }

          if (op == '+')
               msgname = "add";
          else if (op == '-')
               msgname = "sub";
          else if (op == '*')
               msgname = "mul";
          else if (op == '/')
               msgname = "div";
          else
          {
               puts("wrong expression");
               continue;
          }

          cc = yamiCreateParamSet(&paramset, 2);
          if (cc != YAMIOK)
          {
               puts("cannot create parameter set");
               break;
          }

          cc = yamiSetInt(paramset, 0, val1);
          if (cc != YAMIOK)
          {
               puts("cannot set the int value in the parameter set");
               break;
          }

          cc = yamiSetInt(paramset, 1, val2);
          if (cc != YAMIOK)
          {
               puts("cannot set the int value in the parameter set");
               break;
          }

          cc = yamiAgentMsgSend(agent, domainname, objectname,
               msgname, paramset, &hm);
          if (cc != YAMIOK)
          {
               puts("cannot send the message");
               break;
          }

          cc = yamiDestroyParamSet(paramset);
          if (cc != YAMIOK)
          {
               puts("cannot destroy the parameter set");
               break;
          }

          /* set the 10 seconds timeout */
          cc = yamiAgentSetUpTimeOut(hm, 10);
          if (cc != YAMIOK)
          {
               puts("error while setting up timeout");
               break;
          }

          cc = yamiAgentMsgWait(hm);
          if (cc != YAMIOK)
          {
               puts("error while waiting for the response");
               break;
          }

          cc = yamiAgentMsgGetStatus(hm, &status);
          if (cc != YAMIOK)
          {
               puts("cannot retrieve status of the messagege");
               break;
          }

          if (status == eReplied)
          {
               cc = yamiAgentMsgGetResponse(hm, &returnparamset);
               if (cc != YAMIOK)
               {
                    puts("cannot retrieve the return parameter set");
                    break;
               }

               cc = yamiGetInt(returnparamset, 0, &result);
               if (cc != YAMIOK)
               {
                    puts("cannot get int value from the"
                         " parameter set");
                    break;
               }

               cc = yamiDestroyParamSet(returnparamset);
               if (cc != YAMIOK)
               {
                    puts("cannot destroy the return parameter set");
                    break;
               }

               printf("%s = %d\n", buf, result);
          }
          else if (status == eRejected)
          {
               puts("the last message was rejected");
          }
          else if (status == eTimedOut)
          {
               puts("the message was timed out");
               break;
          }
          else
          {
               puts("the last message was not replied correctly");
               break;
          }

          cc = yamiAgentMsgDestroy(hm);
          if (cc != YAMIOK)
          {
               puts("cannot destroy the message object");
               break;
          }
     }

     puts("thank you!");

     cc = yamiDestroyAgent(agent);
     if (cc != YAMIOK)
     {
          puts("cannot destroy the agent");
          return 0;
     }

     cc = yamiNetCleanup();
     if (cc != YAMIOK)
     {
          puts("cannot clean up the network");
          return 0;
     }

     return 0;
}
