#include "yamic.h"
#include "yamiparams.h"
#include "yamisynchro.h"
#include "yamierrors.h"
#include <stdio.h>

const int  serverport  = 12340;
const char *objectname = "calculator";

void servant(HINCMSG incomingmsg)
{
     HPARAMSET paramset, returnparamset;
     const char *msgname;
     int val1, val2, result;
     
     yamiAgentIncomingMsgGetMsgName(incomingmsg,
          &msgname);

     printf("I have received the message: %s\n",
          msgname);

     if (strcmp(msgname, "add") &&
          strcmp(msgname, "sub") &&
          strcmp(msgname, "mul") &&
          strcmp(msgname, "div"))
     {
          puts("unknown name - rejecting");
          yamiAgentIncomingMsgReject(incomingmsg);
          return;
     }

     yamiAgentIncomingMsgGetParameters(incomingmsg,
          &paramset);

     yamiGetInt(paramset, 0, &val1);
     yamiGetInt(paramset, 1, &val2);
          
     yamiDestroyParamSet(paramset);
     
     if (strcmp(msgname, "add") == 0)
     {
          result = val1 + val2;
     }
     else if (strcmp(msgname, "sub") == 0)
     {
          result = val1 - val2;
     }
     else if (strcmp(msgname, "mul") == 0)
     {
          result = val1 * val2;
     }
     else /* msgname is "div" */
     {
          if (val2 == 0)
          {
               puts("dividing by 0 not allowed");
               puts("rejecting");
               yamiAgentIncomingMsgReject(incomingmsg);
               return;
          }
          else
               result = val1 / val2;
     }

     yamiCreateParamSet(&returnparamset, 1);
          
     yamiSetInt(returnparamset, 0, result);
          
     yamiAgentIncomingMsgReply(incomingmsg,
          returnparamset);
          
     yamiDestroyParamSet(returnparamset);
}

int main()
{
     HYAMIAGENT agent;
     
     puts("starting the server");
     
     yamiNetInitialize();
     
     yamiCreateAgent(&agent, serverport, NULL);

     yamiAgentObjectRegister(agent, objectname,
          passive_singlethreaded, servant, NULL);
     
     puts("going to sleep...");
     yamiSleep(0);
     
     yamiDestroyAgent(agent);
     
     yamiNetCleanup();
     
     return 0;
}
