#include "yamic.h"
#include "yamiparams.h"
#include "yamisynchro.h"
#include "yamierrors.h"

#include <stdio.h>

const int  serverport  = 12340;
const char *objectname = "calculator";

void servant(HINCMSG incomingmsg)
{
     HPARAMSET paramset, returnparamset;
     const char *msgname;
     int val1, val2, result;
     int cc;
     
     cc = yamiAgentIncomingMsgGetMsgName(incomingmsg, &msgname);
     if (cc != YAMIOK)
     {
          puts("cannot get the message name");
          return;
     }

     printf("I have received the message: %s\n", msgname);

     if (strcmp(msgname, "add") &&
          strcmp(msgname, "sub") &&
          strcmp(msgname, "mul") &&
          strcmp(msgname, "div"))
     {
          puts("unknown name - rejecting");
          cc = yamiAgentIncomingMsgReject(incomingmsg);
          if (cc != YAMIOK)
          {
               printf("cannot reject");
          }
          return;
     }

     cc = yamiAgentIncomingMsgGetParameters(incomingmsg, &paramset);
     if (cc != YAMIOK)
     {
          puts("cannot retrieve the parameter set");
          return;
     }

     cc = yamiGetInt(paramset, 0, &val1);
     if (cc != YAMIOK)
     {
          puts("cannot retrieve the value");
          return;
     }
     
     cc = yamiGetInt(paramset, 1, &val2);
     if (cc != YAMIOK)
     {
          puts("cannot retrieve the value");
          return;
     }
          
     cc = yamiDestroyParamSet(paramset);
     if (cc != YAMIOK)
     {
          puts("cannot destroy the parameter set");
          return;
     }
     
     if (strcmp(msgname, "add") == 0)
     {
          result = val1 + val2;
     }
     else if (strcmp(msgname, "sub") == 0)
     {
          result = val1 - val2;
     }
     else if (strcmp(msgname, "mul") == 0)
     {
          result = val1 * val2;
     }
     else /* msgname is "div" */
     {
          if (val2 == 0)
          {
               puts("dividing by 0 not allowed - rejecting");
               cc = yamiAgentIncomingMsgReject(incomingmsg);
               if (cc != YAMIOK)
               {
                    puts("cannot reject");
               }
               return;
          }
          else
               result = val1 / val2;
     }

     cc = yamiCreateParamSet(&returnparamset, 1);
     if (cc != YAMIOK)
     {
          puts("cannot create the return parameter set");
          return;
     }
          
     cc = yamiSetInt(returnparamset, 0, result);
     if (cc != YAMIOK)
     {
          puts("cannot set the value in the return parameter set");
          return;
     }
          
     cc = yamiAgentIncomingMsgReply(incomingmsg, returnparamset);
     if (cc != YAMIOK)
     {
          puts("cannot reply to the message");
          return;
     }
          
     cc = yamiDestroyParamSet(returnparamset);
     if (cc != YAMIOK)
     {
          puts("cannot destroy the return parameter set");
          return;
     }
}

int main()
{
     struct yamipolicies policies;
     HYAMIAGENT agent;
     int cc;
     
     puts("starting the server");
     
     cc = yamiNetInitialize();
     if (cc != YAMIOK)
     {
          puts("cannot initialize network");
          return 0;
     }
     
     /* prepare policies for explicit events processing */
     yamiGetDefaultPolicies(&policies);
     policies.dispatchers = 0;
     policies.hassender = 0;
     policies.hasreceiver = 0;
     policies.haswaker = 0;
     policies.mtuser = 0;

     cc = yamiCreateAgent(&agent, serverport, &policies);
     if (cc != YAMIOK)
     {
          puts("cannot create the agent");
          return cc;
     }

     cc = yamiAgentObjectRegister(agent, objectname,
          passive_singlethreaded, servant, NULL);
     if (cc != YAMIOK)
     {
          puts("cannot register the object");
          return cc;
     }
     
     puts("running event loop...");

     while (1)
     {
          /* perform blocking processing - the function will wait     */
          /* for the connection, which may be a message to the server */

          cc = yamiAgentProcessEvent(agent, -1);
          if (cc != YAMIOK)
          {
               puts("error while processing event");
               break;
          }
     }

     /* this code should never execute... */

     cc = yamiDestroyAgent(agent);
     if (cc != YAMIOK)
     {
          puts("cannot destroy the agent");
          return cc;
     }
     
     cc = yamiNetCleanup();
     if (cc != YAMIOK)
     {
          puts("cannot clean up the network");
          return 0;
     }
     
     return 0;
}
