#include "yamic.h"
#include "yamiparams.h"
#include "yamisynchro.h"
#include "yamierrors.h"
#include <stdio.h>
#include <string.h>

HSEMAPHORE sem;

void servant(HINCMSG incoming)
{
     HPARAMSET params;
     const char *msgname;
     const char *line;
     int cc;

     cc = yamiAgentIncomingMsgGetMsgName(incoming, &msgname);
     if (cc != YAMIOK)
     {
          puts("cannot get message name");
          return;
     }

     if (strcmp(msgname, "shutdown") == 0)
     {
          puts("received the shutdown message");
          cc = yamiSemaphoreRelease(sem);
          if (cc != YAMIOK)
          {
               puts("cannot release semaphore");
               return;
          }
     }
     else /* if (strcmp(msgname, "print") == 0) */
     {
          cc = yamiAgentIncomingMsgGetParameters(incoming,
               &params);
          if (cc != YAMIOK)
          {
               puts("cannot get parameters");
               return;
          }
          
          cc = yamiGetStringBuffer(params, 0, &line);
          if (cc != YAMIOK)
          {
               puts("cannot read string buffer");
               return;
          }

          puts(line);
               
          cc = yamiDestroyParamSet(params);
          if (cc != YAMIOK)
          {
               puts("cannot destroy paramset");
               return;
          }
     }
     
     cc = yamiAgentIncomingMsgEat(incoming);
     if (cc != YAMIOK)
     {
          puts("cannot eat message");
          return;
     }
}

int main()
{
     struct yamipolicies policies;
     HYAMIAGENT agent;
     int cc;

     cc = yamiNetInitialize();
     if (cc != YAMIOK)
     {
          puts("cannot initialize network");
          return -1;
     }
     
     cc = yamiCreateSemaphore(&sem, 0);
     if (cc != YAMIOK)
     {
          puts("cannot create semaphore");
          return -1;
     }
     
     yamiGetDefaultPolicies(&policies);
     policies.dispatchers = 0;
     policies.reuseaddr = 1;
          
     cc = yamiCreateAgent(&agent, 12340, &policies);
     if (cc != YAMIOK)
     {
          puts("cannot create agent");
          return -1;
     }

     cc = yamiAgentObjectRegister(agent, "echo",
          passive_singlethreaded, servant, NULL);
     if (cc != YAMIOK)
     {
          puts("cannot register object");
          return -1;
     }

     puts("server started");
          
     cc = yamiSemaphoreAcquire(sem);
     if (cc != YAMIOK)
     {
          puts("cannot acquire semaphore");
          return -1;
     }
     
     /* the process reaches this code       */
     /* only when the semaphore is released */
          
     cc = yamiDestroyAgent(agent);
     if (cc != YAMIOK)
     {
          puts("cannot destroy agent");
          return -1;
     }
     
     cc = yamiDestroySemaphore(sem);
     if (cc != YAMIOK)
     {
          puts("cannot destroy semaphore");
          return -1;
     }
     
     cc = yamiNetCleanup();
     if (cc != YAMIOK)
     {
          puts("cannot cleanup network");
          return -1;
     }
     
     return 0;
}
