#include "yami++.h"
#include <iostream>

using namespace YAMI;
using namespace std;

const int  serverport  = 12340;
const char *objectname = "calculator";

class Server : public PassiveObject
{
public:
     void call(IncomingMsg &incoming)
     {
          try
          {
               string msgname(incoming.getMsgName());

               cout << "I have received the message: "
                    << msgname << endl;

               if (msgname != "add" &&
                    msgname != "sub" &&
                    msgname != "mul" &&
                    msgname != "div")
               {
                    cout << "unknown name - rejecting" << endl;
                    incoming.reject();
                    return;
               }

               auto_ptr<ParamSet> params(incoming.getParameters());
     
               int val1 = params->getInt(0);
               int val2 = params->getInt(1);
          
               int result;
               if (msgname == "add")
               {
                    result = val1 + val2;
               }
               else if (msgname == "sub")
               {
                    result = val1 - val2;
               }
               else if (msgname == "mul")
               {
                    result = val1 * val2;
               }
               else /* msgname is "div" */
               {
                    if (val2 == 0)
                    {
                         cout << "dividing by 0 not allowed"
                              << endl << "rejecting" << endl;
                         incoming.reject();
                         return;
                    }
                    else
                         result = val1 / val2;
               }

               ParamSet returnparams(1);
               returnparams.setInt(0, result);

               incoming.reply(returnparams);
          }
          catch (const exception &e)
          {
               cout << "exception: " << e.what() << endl;
          }
     }
};

int main()
{
     cout << "starting the server" << endl;
     try
     {
          netInitialize();
          {
               Server servant;
               Agent agent(serverport);
               agent.objectRegister(objectname,
                    Agent::ePassiveSingleThreaded,
                    &servant);

               cout << "going to sleep..." << endl;
               sleep(0);
          }
          netCleanup();
     }
     catch (const exception &e)
     {
          cout << "exception: " << e.what() << endl;
     }
     
     return 0;
}
