#include "yami++.h"
#include <iostream>
#include <string>

using namespace YAMI;
using namespace std;

// for sscanf
#include <stdio.h>

const char *serverhost  = "127.0.0.1";
const int  serverport   = 12340;
const int  clientport   = 12341;
const char *domainname  = "someDomain";
const char *objectname  = "calculator";

int main()
{
     cout << "starting the client" << endl;

     netInitialize();
     {
          // prepare policies for explicit events processing
          Policies policies;
          policies.setDispatchers(0);
          policies.setHasSender(false);
          policies.setHasReceiver(false);
          policies.setHasWaker(false);
          policies.setMTUser(false);

          Agent agent(clientport, policies);
          agent.domainRegister(domainname,
               serverhost, serverport, 2);

          while (1)
          {
               cout << "please enter the expression (or \'quit\'): ";

               string str;
               getline(cin, str);

               if (str == "quit")
                    break;

               // note: I use sscanf here, because
               // istringstream is not everywhere available

               int val1, val2;
               char op;

               if (sscanf(str.c_str(), "%d%c%d", &val1, &op, &val2) != 3)
               {
                    cout << "wrong expression" << endl;
                    continue;
               }

               string msgname;
               if (op == '+')
                    msgname = "add";
               else if (op == '-')
                    msgname = "sub";
               else if (op == '*')
                    msgname = "mul";
               else if (op == '/')
                    msgname = "div";
               else
               {
                    cout << "wrong expression" << endl;
                    continue;
               }

               ParamSet paramset(2);
               paramset.setInt(0, val1);
               paramset.setInt(1, val2);

               auto_ptr<Message> msg(agent.send(domainname, objectname,
                    msgname.c_str(), paramset));


               // perform the explicit message loop

               while (true)
               {
                    // do blocking processing - the function will wait
                    // for the incoming connection, which may be a reply

                    agent.processEvent(-1);

                    // check the status of the message

                    Message::eStatus status = msg->getStatus();

                    // continue processing if there is no change

                    if (status == Message::ePending)
                         continue;

                    if (status == Message::eReplied)
                    {
                         auto_ptr<ParamSet> retpar(msg->getResponse());

                         int result = retpar->getInt(0);

                         cout << str << " = " << result << endl;
                         break;
                    }
                    else if (status == Message::eRejected)
                    {
                         cout << "the last message was rejected" << endl;
                         break;
                    }
                    else
                    {
                         cout << "the last message was not replied correctly"
                              << endl;
                         break;
                    }
               }
          }
     }

     cout << "thank you!" << endl;

     netCleanup();

     return 0;
}
