#include "yami++.h"
#include <iostream>

using namespace YAMI;
using namespace std;

const int  serverport  = 12340;
const char *objectname = "calculator";

class Server : public PassiveObject
{
public:
     void call(IncomingMsg &incoming)
     {
          try
          {
               string msgname(incoming.getMsgName());

               cout << "I have received the message: "
                    << msgname << endl;

               if (msgname != "add" &&
                    msgname != "sub" &&
                    msgname != "mul" &&
                    msgname != "div")
               {
                    cout << "unknown name - rejecting" << endl;
                    incoming.reject();
                    return;
               }

               auto_ptr<ParamSet> params(incoming.getParameters());
     
               int val1 = params->getInt(0);
               int val2 = params->getInt(1);
          
               int result;
               if (msgname == "add")
               {
                    result = val1 + val2;
               }
               else if (msgname == "sub")
               {
                    result = val1 - val2;
               }
               else if (msgname == "mul")
               {
                    result = val1 * val2;
               }
               else /* msgname is "div" */
               {
                    if (val2 == 0)
                    {
                         cout << "dividing by 0 not allowed"
                              << endl << "rejecting" << endl;
                         incoming.reject();
                         return;
                    }
                    else
                         result = val1 / val2;
               }

               ParamSet returnparams(1);
               returnparams.setInt(0, result);

               incoming.reply(returnparams);
          }
          catch (const exception &e)
          {
               cout << "exception: " << e.what() << endl;
          }
     }
};

int main()
{
     cout << "starting the server" << endl;
     try
     {
          netInitialize();
          {
               // prepare policies for explicit events processing
               Policies policies;
               policies.setDispatchers(0);
               policies.setHasSender(false);
               policies.setHasReceiver(false);
               policies.setHasWaker(false);
               policies.setMTUser(false);

               Server servant;
               Agent agent(serverport, policies);
               agent.objectRegister(objectname,
                    Agent::ePassiveSingleThreaded,
                    &servant);

               cout << "running event loop..." << endl;

               while (true)
               {
                    // perform blocking processing - the function will wait
                    // for the connection, which may be a message to server

                    agent.processEvent(-1);
               }
          }
          netCleanup();
     }
     catch (const exception &e)
     {
          cout << "exception: " << e.what() << endl;
     }
     
     return 0;
}
