#include "yami++.h"
#include <iostream>
#include <string>

using namespace YAMI;
using namespace std;

class Server : public PassiveObject
{
public:
     Server(Semaphore &s) : sem_(s) {}

     void call(IncomingMsg &incoming)
     {
          try
          {
               string msgname(incoming.getMsgName());
               if (msgname == "shutdown")
               {
                    cout << "received the shutdown message"
                         << endl;
                    sem_.release();
               }
               else // if (msgname == "print")
               {
                    auto_ptr<ParamSet> params
                         (incoming.getParameters());

                    string line;
                    params->getString(0, line);
                    cout << line << endl;
               }

               incoming.eat();
          }
          catch (const exception &e)
          {
               cout << "exception in call: " << e.what()
                    << endl;
          }
     }

private:
     Semaphore &sem_;
};


int main()
{
     try
     {
          netInitialize();
          {
               Semaphore sem(0);
               Server servant(sem);

               Policies policies;
               policies.setDispatchers(0);
               policies.setReuseAddr(true);

               Agent agent(12340, policies);
               agent.objectRegister("echo",
                    Agent::ePassiveSingleThreaded,
                    &servant);

               cout << "server started" << endl;

               sem.acquire();

               // the process quits this block
               // only when the semaphore is released
          }
          netCleanup();
     }
     catch (const exception &e)
     {
          cout << "exception: " << e.what() << endl;
     }
     
     return 0;
}
