#!/usr/local/bin/wish8.3 -f
#
# Note: the above path may be different on your Unix system
#
# psConsole.tcl
#
# this file implements the GUI console to the Property Service
#
# Copyright (C) 2001-2003 Maciej Sobczak,
# student at the Institute of Computer Science,
# Warsaw University of Technology
#
# Permission to copy, use, modify, sell and distribute this software
# is granted provided this copyright notice appears in all copies.
# This software is provided "as is" without express or implied
# warranty, and with no claim as to its suitability for any purpose.
#

package require YAMI

toplevel .dialog

frame .dialog.le

frame .dialog.labels
label .dialog.addrl -text "enter the server's address: "
label .dialog.portl -text "enter the server's port: "
label .dialog.locportl -text "choose the local port: "

frame .dialog.entries
entry .dialog.addre -bg white -textvariable ps_address
entry .dialog.porte -bg white -validate all \
	-vcmd { string is integer %P } -textvariable ps_port
entry .dialog.locporte -bg white -validate all \
	-vcmd { string is integer %P } -textvariable ps_local_port

button .dialog.connect -text "connect" -command ps_connect

pack .dialog.le
pack .dialog.labels .dialog.entries -in .dialog.le -side left
pack .dialog.addrl .dialog.portl .dialog.locportl \
	-in .dialog.labels -anchor w -padx 2 -pady 2
pack .dialog.addre .dialog.porte .dialog.locporte \
	-in .dialog.entries -padx 2 -pady 2
pack .dialog.connect -padx 2 -pady 2

grab set .dialog

proc ps_connect {} {
	global agent ps_address ps_port ps_local_port
	
	set agent [YAMI::Agent $ps_local_port]
	$agent domainRegister ps $ps_address $ps_port 2

	reload_objects 1

	grab release .dialog
	destroy .dialog
}

# this is the main part of the console

# list of objects

frame .fobj
label .fobj.l -text "objects on the server:"
frame .fobj.fobjs
scrollbar .fobj.fobjs.scroll -command ".fobj.fobjs.list yview"
listbox .fobj.fobjs.list -yscroll ".fobj.fobjs.scroll set" \
	-setgrid 1 -height 6
frame .fobj.fo
label .fobj.fo.l -anchor w -width 10 -text "object "
entry .fobj.fo.e -bg white -textvariable ps_object_name
button .fobj.fo.new -width 4 -text "new" \
	-command { ps_new_object }
button .fobj.fo.remove -text "remove" \
	-command { ps_remove_object }

pack .fobj -fill x
pack .fobj.l -anchor w -padx 2 -pady 2
pack .fobj.fobjs -fill x -padx 2 -pady 2
pack .fobj.fobjs.scroll -side right -fill y
pack .fobj.fobjs.list -side left -expand 1 -fill both
pack .fobj.fo -fill x -padx 2 -pady 2
pack .fobj.fo.l -side left
pack .fobj.fo.e -side left -fill x -expand 1
pack .fobj.fo.new .fobj.fo.remove -side left

bind .fobj.fobjs.list <Double-1> {
	set ps_object_name [selection get]
	reload_properties 1
}

# list of properties

frame .fpr
label .fpr.l -text "properties:"
frame .fpr.fprs
scrollbar .fpr.fprs.scroll -command ".fpr.fprs.list yview"
listbox .fpr.fprs.list -yscroll ".fpr.fprs.scroll set" \
	-setgrid 1 -height 6
frame .fpr.fp
label .fpr.fp.l -anchor w -width 10 -text "property "
entry .fpr.fp.e -bg white -textvariable ps_property_name
button .fpr.fp.set -width 4 -text "set" \
	-command { ps_set_property }
button .fpr.fp.remove -text "remove" \
	-command { ps_remove_property }
frame .fpr.ft
label .fpr.ft.l -anchor w -width 10 -text "type "
radiobutton .fpr.ft.string -text "string" \
	-variable ps_type -value s
radiobutton .fpr.ft.int -text "int" \
	-variable ps_type -value i
radiobutton .fpr.ft.double -text "double" \
	-variable ps_type -value d
radiobutton .fpr.ft.byte -text "byte" \
	-variable ps_type -value o
#radiobutton .fpr.ft.binary -text "binary" \
#	-variable ps_type -value b
frame .fpr.fv
label .fpr.fv.l -anchor w -width 10 -text "value "
entry .fpr.fv.value -bg white -textvariable ps_value

pack .fpr -fill x -pady 2
pack .fpr.l -anchor w -padx 2 -pady 2
pack .fpr.fprs -fill x -padx 2 -pady 2
pack .fpr.fprs.scroll -side right -fill y
pack .fpr.fprs.list -side left -expand 1 -fill both
pack .fpr.fp .fpr.ft .fpr.fv -fill x -padx 2 -pady 2
pack .fpr.fp.l -side left
pack .fpr.fp.e -side left -fill x -expand 1
pack .fpr.fp.set .fpr.fp.remove -side left
pack .fpr.ft.l .fpr.ft.string .fpr.ft.int \
	.fpr.ft.double .fpr.ft.byte -side left
pack .fpr.fv.l -side left
pack .fpr.fv.value -side left -fill x -expand 1

bind .fpr.fprs.list <Double-1> {
	set ps_property_name [selection get]

	# get the property value
	set m [$agent send ps $ps_object_name get $ps_property_name]
	$m wait
	set s [$m getStatus]
	if {$s == "eReplied"} {
		set resp [$m getResponse 1]
		set value [lindex $resp 0]
		set ps_type [lindex $value 1]
		if {$ps_type == "w"} {
			set ps_value "-- wide string data --"
		} elseif {$ps_type == "b"} {
			set ps_value "-- binary data --"
		} else {
			set ps_value [lindex $value 0]
		}
	}
	
	status $s
	
	$m -delete
}

# general controls

frame .fgen -relief groove -border 3
frame .fgen.b
label .fgen.b.l -text "server operations:"
button .fgen.b.write -text "write data to file" -command ps_write_data
button .fgen.b.quit -text "close the server" -command ps_quit
frame .fgen.fstat
label .fgen.fstat.l -text "status: "
label .fgen.fstat.s -text ""

pack .fgen -fill x -expand 1
pack .fgen.b .fgen.fstat -anchor w
pack .fgen.b.l .fgen.b.write .fgen.b.quit -side left -padx 2 -pady 2
pack .fgen.fstat.l .fgen.fstat.s -side left -padx 2 -pady 2

# procedures

# get all the object names
proc reload_objects {display_status} {
	global agent
	
	set m [$agent send ps propertyserver list]
	$m wait
	set s [$m getStatus]
	if {$s == "eReplied"} {
		.fobj.fobjs.list delete 0 end
		eval [concat .fobj.fobjs.list insert 0 [$m getResponse]]
	}
	
	if {$display_status == 1} {
		status $s
	}
	
	$m -delete
}

# create new object on the server
proc ps_new_object {} {
	global agent ps_object_name
	
	set m [$agent send ps propertyserver new $ps_object_name]
	$m wait
	status [$m getStatus]
	$m -delete
	
	reload_objects 0
}

# remove the object from the server
proc ps_remove_object {} {
	global agent ps_object_name
	
	set m [$agent send ps propertyserver remove $ps_object_name]
	$m wait
	status [$m getStatus]
	$m -delete
	
	reload_objects 0
}

# get all the property names for the selected object
proc reload_properties {display_status} {
	global agent ps_object_name
	
	set m [$agent send ps $ps_object_name list]
	$m wait
	set s [$m getStatus]
	if {$s == "eReplied"} {
		.fpr.fprs.list delete 0 end
		eval [concat .fpr.fprs.list insert 0 [$m getResponse]]
	}

	if {$display_status == 1} {	
		status $s
	}
	
	$m -delete
}

# set the property's value for the selected object
proc ps_set_property {} {
	global agent ps_object_name ps_property_name ps_value ps_type
	
	set m [$agent send ps $ps_object_name set \
		[list [list $ps_property_name s] [list $ps_value $ps_type]]]
	$m wait
	status [$m getStatus]
	$m -delete
	
	reload_properties 0
}

# remove the property for the selected object
proc ps_remove_property {} {
	global agent ps_object_name ps_property_name
	
	set m [$agent send ps $ps_object_name remove $ps_property_name]
	$m wait
	status [$m getStatus]
	$m -delete
	
	reload_properties 0
}

# write all the server's data to its working file
proc ps_write_data {} {
	global agent
	
	set m [$agent send ps propertyserver write]
	$m -delete
	status "data written"
}

# close down the server
proc ps_quit {} {
	global agent
	
	set m [$agent send ps propertyserver_quit quit]
	$m -delete
	status "server closed down"
}

# helper procedure

# display the status in the window
proc status {s} {
	.fgen.fstat.s configure -text $s
}

# initial settings

set ps_type s
status "not connected"
