//
// cppgen.h
//
// definition of the C++ generator's interface
//
// Copyright (C) 2001-2003 Maciej Sobczak,
// student at the Institute of Computer Science,
// Warsaw University of Technology
//
// Permission to copy, use, modify, sell and distribute this software
// is granted provided this copyright notice appears in all copies.
// This software is provided "as is" without express or implied
// warranty, and with no claim as to its suitability for any purpose.
//

#ifndef CPPGENERATOR_INCLUDED
#define CPPGENERATOR_INCLUDED

#include "ydl.h"
#include <string>

class CPPGenerator : public Generator
{
public:
     CPPGenerator() {}
     
     void generate(const YDL &ydl, const std::string &corename,
          const options &opts, std::ostream &err);

private:
     // copyright
     CPPGenerator(const CPPGenerator &);
     CPPGenerator & operator=(const CPPGenerator &);
     
     enum eBinStrForm { ePointer, eStdString, eStdVector, eMalloc, eNew };

     void generateClientHeader(const YDL &ydl, std::ostream &os,
          const std::string &namespc, const std::string &headername,
          eBinStrForm stringinform, eBinStrForm stringoutform,
          eBinStrForm bininform, eBinStrForm binoutform,
          std::ostream &err);
     void generateClientDefinitions(const YDL &ydl, std::ostream &os,
          const std::string &namespc, const std::string &filename,
          eBinStrForm stringinform, eBinStrForm stringoutform,
          eBinStrForm bininform, eBinStrForm binoutform,
          std::ostream &err);
     void generateServerHeader(const YDL &ydl, std::ostream &os,
          const std::string &namespc, const std::string &filename,
          eBinStrForm stringinform, eBinStrForm stringoutform,
          eBinStrForm bininform, eBinStrForm binoutform,
          std::ostream &err);
     void generateServerDefinitions(const YDL &ydl, std::ostream &os,
          const std::string &namespc, const std::string &headername,
          eBinStrForm stringinform, eBinStrForm stringoutform,
          eBinStrForm bininform, eBinStrForm binoutform,
          std::ostream &err);
     
     std::string getUpper(const std::string &name);
     void doIndent(std::ostream &os, int indent);
     void generateParams(const Message &msg, std::ostream &os,
          eBinStrForm stringinform, eBinStrForm stringoutform,
          eBinStrForm bininform, eBinStrForm binoutform,
          int indent, std::ostream &err);
     void generateParamSetFeedCli(std::ostream &os,
          const std::vector<Param> &params, const char *paramsname,
          eBinStrForm stringinform, eBinStrForm bininform,
          std::ostream &err);
     void generateParamSetReadCli(std::ostream &os,
          const std::vector<Param> &params, const char *paramsname,
          eBinStrForm stringoutform, eBinStrForm binoutform,
          std::ostream &err);
     void generateParamSetFeedSvr(std::ostream &os,
          const std::vector<Param> &params, const char *paramsname,
          eBinStrForm stringoutform, eBinStrForm binoutform,
          std::ostream &err);
     void generateParamSetReadSvr(std::ostream &os,
          const std::vector<Param> &params, const char *paramsname,
          eBinStrForm stringinform, eBinStrForm bininform,
          std::ostream &err);
     void generateSizeDecl(std::ostream &os, bool &alreadyGenerated);
     void generateHandlersSvr(std::ostream &os,
          const std::vector<Param> &params,
          eBinStrForm stringoutform, eBinStrForm binoutform,
          std::ostream &err);
     void generateHandlers(std::ostream &os, std::ostream &err);
};

#endif
