//
// main.cc
//
// main file for the YDL tool
//
// Copyright (C) 2001-2003 Maciej Sobczak,
// student at the Institute of Computer Science,
// Warsaw University of Technology
//
// Permission to copy, use, modify, sell and distribute this software
// is granted provided this copyright notice appears in all copies.
// This software is provided "as is" without express or implied
// warranty, and with no claim as to its suitability for any purpose.
//

#include <iostream>
#include <fstream>
#include <vector>
#include <string>
#include <cstdlib>

#include "ydl.h"
#include "parser.h"
#include "cppgen.h"

using namespace std;

typedef vector<string> strings;

Generator & getGenerator(const options &opts)
{
     options::const_iterator it = opts.find("language");
     if (it == opts.end())
     {
          cerr << "ydl: The language is not specified\n";
          throw 0;
     }
     
     if (it->second == "cpp")
     {
          static CPPGenerator generator;
          return generator;
     }
     else // put additional generators here in the if-else chain
     {
          cerr << "ydl: The language is unknown\n";
          throw 0;
     }
}

int main(int argc, char *argv[])
{
     strings filenames;
     options opts;
     
     for (int i = 1; i != argc; ++i)
     {
          if (argv[i][0] == '-')
          {
               // option
               string optname(argv[i] + 1);
               if (i < argc - 1)
               {
                    ++i;
                    string optvalue(argv[i]);
                    opts[optname] = optvalue;
               }
               else
               {
                    cerr << "ydl: Option " << optname << " without value\n";
                    return EXIT_FAILURE;
               }
          }
          else
          {
               // name of the ydl file
               filenames.push_back(argv[i]);
          }
     }
     
     if (filenames.size() == 0)
     {
          // no input file - reading from cin
          // but only if there is a core name defined
          options::iterator i = opts.find("name");
          if (i == opts.end())
          {
               cerr << "ydl: No input file and no name defined\n";
               return EXIT_FAILURE;
          }

          string corename(i->second);

          try
          {
               Parser parser(cin, cerr);

               Generator &generator = getGenerator(opts);
               generator.generate(parser.getDefinitions(),
                    corename, opts, cerr);
          }
          catch (...)
          {
               cerr << "ydl: Cannot proceed\n";
               return EXIT_FAILURE;
          }
     }
     else
     {
          for (strings::iterator name = filenames.begin();
               name != filenames.end(); ++name)
          {
               string corename(*name);
               string::size_type pos = corename.find_last_of(".");
               if (pos != string::npos)
                    corename.erase(pos);
               
               ifstream file(name->c_str());
               if (file)
               {
                    try
                    {
                         Parser parser(file, cerr);
                         
                         Generator &generator = getGenerator(opts);
                         generator.generate(parser.getDefinitions(),
                              corename, opts, cerr);
                    }
                    catch (...)
                    {
                         cerr << "ydl: Cannot proceed\n";
                         return EXIT_FAILURE;
                    }
               }
               else
               {
                    cerr << "ydl: " << *name << ": Cannot open file\n";
                    return EXIT_FAILURE;
               }
          }
     }

     return EXIT_SUCCESS;
}
