//
// parser.h
//
// definition of the parser's interface
//
// Copyright (C) 2001-2003 Maciej Sobczak,
// student at the Institute of Computer Science,
// Warsaw University of Technology
//
// Permission to copy, use, modify, sell and distribute this software
// is granted provided this copyright notice appears in all copies.
// This software is provided "as is" without express or implied
// warranty, and with no claim as to its suitability for any purpose.
//

#ifndef PARSER_INCLUDED
#define PARSER_INCLUDED

#include "ydl.h"
#include <iostream>

class Parser
{
public:
     Parser(std::istream &input, std::ostream &errors);
     const YDL & getDefinitions() const { return ydl_; }

private:

     // token definitions
     static const char *opentoken[];
     static const char *closetoken[];
     static const char *intoken[];
     static const char *outtoken[];
     static const char *typetoken[];
     static const char *onewaytoken[];
     static const char *endtoken[];
     static const char *commenttoken[];

     // tokens
     enum eToken { eName, eOpen, eClose, eIn, eOut, eOneWay, eEnd };
     
     bool beginsWith(std::string::iterator i1, std::string::iterator i2,
          const char *p);
     bool getParams(std::vector<Param> &params);
     bool getInOutOneWayEnd(Message &msg);
     bool getOutOneWayEnd(Message &msg);
     bool getOneWayEnd(Message &msg);
     bool getInterface();
     void printMsg(const char *msg);
     void showError();
     bool readLine();
     bool getNextChar(bool skipnewlines);
     bool skipWhites();
     bool getName();
     bool getOpen();
     bool getClose();
     bool getIn();
     bool getOut();
     bool getType();
     bool getOneWay();
     bool getEnd();
     
     // copyright
     Parser(const Parser &);
     Parser & operator=(const Parser &);
     
     std::string currentline_;
     int linenumber_;
     std::string::iterator p_;     // current character in currentline_
     std::string currenttoken_;
     std::istream &in_;
     std::ostream &err_;
     YDL ydl_;
};

#endif
