//
// ydl.h
//
// definition of the YDL structure
//
// Copyright (C) 2001-2003 Maciej Sobczak,
// student at the Institute of Computer Science,
// Warsaw University of Technology
//
// Permission to copy, use, modify, sell and distribute this software
// is granted provided this copyright notice appears in all copies.
// This software is provided "as is" without express or implied
// warranty, and with no claim as to its suitability for any purpose.
//

#ifndef YDL_INCLUDED
#define YDL_INCLUDED

#include <vector>
#include <map>
#include <string>

enum eType
{
     eString,
     eWString,
     eInt,
     eDouble,
     eByte,
     eBinary
};

struct Param
{
     Param(eType t, const std::string &n) : type(t), name(n) {}
     
     eType type;
     std::string name;
};

struct Message
{
     Message(const std::string &n) : name(n), oneway(false) {}
     
     std::string name;
     std::vector<Param> input;
     std::vector<Param> output;
     bool oneway;
};

struct Interface
{
     Interface(const std::string &n) : name(n) {}
     
     std::string name;
     std::vector<Message> messages;
};

typedef std::vector<Interface> YDL;

typedef std::map<std::string, std::string> options;

class Generator
{
public:
     virtual void generate(const YDL &ydl, const std::string &corename,
          const options &opts, std::ostream &err) = 0;
};

#endif
